﻿/* Мною, Константином aka KilkennyCat, 05 июля 2020 года создано сиё 
 * на основе "Описание формата TopoR PCB версия 1.2.0 Апрель 2017 г.".
 * k@kilkennycat.pro
 * http://kilkennycat.ru  http://kilkennycat.pro
 */

using System.Collections.Generic;
using System.Linq;
using System.Xml.Serialization;

namespace TopoR_PCB_Classes
{
    /// <summary>
    /// Раздел «Правила для высокоскоростных устройств».
    /// </summary>
    public class HiSpeedRules
    {
        /// <summary>
        /// Волновое сопротивление и правила разводки сигналов по слоям.
        /// </summary>
        public class Impedance
        {
            public class LayerRule_Impendance
            {
                /// <summary>
                /// Ширина проводника.
                /// </summary>
                [XmlAttribute("width", DataType = "float")]
                public float _width;

                /// <summary>
                /// Ссылка на слой.
                /// </summary>
                [XmlElement("LayerRef")]
                public LayerRef _LayerRef;
            }

            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Параметр правила разводки дифференциальной пары: значение волнового сопротивления в Омах.
            /// </summary>
            [XmlAttribute("Z0", DataType = "float")]
            public float _z0;

            /// <summary>
            /// Правило разводки сигнала для слоя.
            /// </summary>
            [XmlElement("LayerRule")]
            public List<LayerRule_Impendance> _LayerImpedanceRules;
            public bool ShouldSerialize_LayerImpedanceRules()
            {
                return _LayerImpedanceRules?.Count > 0;
            }
        }

        /// <summary>
        /// Волновое сопротивление и правила разводки сигналов по слоям для дифференциальных сигналов.
        /// </summary>
        public class ImpedanceDiff
        {
            public class LayerRule_ImpendanceDiff
            {
                /// <summary>
                /// Ширина проводника.
                /// </summary>
                [XmlAttribute("width", DataType = "float")]
                public float _width;

                /// <summary>
                /// Параметр правила разводки дифференциальных пар: зазор между проводниками пары.
                /// </summary>
                [XmlAttribute("gap", DataType = "float")]
                public float _gap;

                /// <summary>
                /// Ссылка на слой.
                /// </summary>
                [XmlElement("LayerRef")]
                public LayerRef _LayerRef;
            }

            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Параметр правила разводки дифференциальной пары: значение волнового сопротивления в Омах.
            /// </summary>
            [XmlAttribute("Z0", DataType = "float")]
            public float _z0;

            /// <summary>
            /// Правило разводки дифференциальной пары для слоя.
            /// </summary>
            [XmlElement("LayerRule")]
            public List<LayerRule_ImpendanceDiff> _LayerImpedanceDiffRules;
            public bool ShouldSerialize_LayerImpedanceDiffRules()
            {
                return _LayerImpedanceDiffRules?.Count > 0;
            }
        }

        /// <summary>
        /// Описание сигнального кластера цепей.
        /// </summary>
        public class SignalCluster
        {
            /// <summary>
            /// Описание заданной связи.
            /// </summary>
            public class PinPair
            {
                /// <summary>
                /// Ссылка на контакт источника сигнала.
                /// </summary>
                [XmlElement("PinRef")]
                public List<PinRef> _PinRefs;
                public bool ShouldSerialize_PinRefs()
                {
                    return _PinRefs?.Count > 0;
                }
            }

            /// <summary>
            /// Описание сигнала.
            /// </summary>
            public class Signal
            {
                /// <summary>
                /// Имя объекта или ссылка на именованный объект.
                /// </summary>
                [XmlAttribute("name")]
                public string _name;

                /// <summary>
                /// Ссылка на контакт источника сигнала.
                /// </summary>
                [XmlElement("ReceiverPinRef")]
                public ReceiverPinRef _ReceiverPinRef;

                /// <summary>
                /// Пассивные компоненты на пути следования сигнала.
                /// </summary>
                [XmlArray("Components")]
                [XmlArrayItem("CompInstanceRef")]
                public List<CompInstanceRef> _Components;
             /*   public bool ShouldSerialize_Components()
                {
                    return _Components?.Count > 0;
                }*/
            }

            /// <summary>
            /// Ссылка на волновое сопротивление.
            /// </summary>
            [XmlElement("ImpedanceRef")]
            public ImpedanceRef _ImpedanceRef;

            /// <summary>
            /// Ссылка на контакт источника сигнала.
            /// </summary>
            [XmlElement("SourcePinRef")]
            public SourcePinRef _SourcePinRef;

            /// <summary>
            /// Цепи сигнального кластера.
            /// </summary>
            [XmlArray("Nets")]
            [XmlArrayItem("NetRef")]
            public List<NetRef> _Nets;
       /*     public bool ShouldSerialize_Nets()
            {
                return _Nets?.Count > 0;
            }
       */
            /// <summary>
            /// Описание заданных связей сигнального кластера.
            /// </summary>
            [XmlArray("PinPairs")]
            [XmlArrayItem("PinPair")]
            public List<PinPair> _PinPairs;

            /// <summary>
            /// Ссылки на сигналы.
            /// </summary>
            [XmlElement("Signal")]
            public List<Signal> _Signals;
        }

        /// <summary>
        /// Описание дифференциального сигнала (дифференциальной пары).
        /// </summary>
        public class DiffSignal
        {
            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Параметр дифференциальной пары: допустимый разброс длины между проводниками пары.
            /// </summary>
            [XmlAttribute("mismatch", DataType = "float")]
            public float _mismatch;

            /// <summary>
            /// Ссылка на волновое сопротивление.
            /// </summary>
            [XmlElement("ImpedanceRef")]
            public ImpedanceRef _ImpedanceRef;

            /// <summary>
            /// Ссылки на сигналы.
            /// </summary>
            [XmlElement("SignalRef")]
            public List<SignalRef> _SignalRefs;
            public bool ShouldSerialize_SignalRefs()
            {
                return _SignalRefs?.Count > 0;
            }
        }

        /// <summary>
        /// Описание группы сигналов.
        /// </summary>
        public class SignalGroup
        {
            /// <summary>
            /// Имя объекта или ссылка на именованный объект.
            /// </summary>
            [XmlAttribute("name")]
            public string _name;

            /// <summary>
            /// Ссылки на сигнал, диф.сигнал, или группу сигналов
            /// </summary>
            /// <value>SignalRef, DiffSignalRef, SignalGroupRef</value>
            [
                XmlElement("SignalRef", typeof(SignalRef)),
                XmlElement("DiffSignalRef", typeof(DiffSignalRef)),
                XmlElement("SignalGroupRef", typeof(SignalGroupRef))
            ]
            public List<object> _References;
            public bool ShouldSerialize_References()
            {
                return _References?.Count > 0;
            }
        }

        /// <summary>
        /// Описание правил выравнивания задержек.
        /// </summary>
        public class RulesDelay
        {
            /// <summary>
            /// Описание правила выравнивания задержек для группы цепей или группы дифференциальных пар.
            /// </summary>
            public class DelayEqual
            {
                /// <summary>
                /// Флаг применения правила.
                /// </summary>
                [XmlAttribute("enabled")]
                public Bool _enabled;
                [XmlIgnore]
                public bool _enabledSpecified { get { return _enabled != Bool.off; } }

                /// <summary>
                /// Параметр правил выравнивания задержек: тип значений констант и допусков.
                /// </summary>
                [XmlAttribute("valueType")]
                public valueType _valueType;

                /// <summary>
                /// Параметр правила выравнивания задержек внутри группы цепей: допуск.
                /// </summary>
                /// <remarks>! Единицы измерения значения зависят от параметра valueType и единиц заданных для всего файла(см.Units).</remarks>
                [XmlAttribute("tolerance", DataType = "float")]
                public float _tolerance;

                /// <summary>
                /// Объекты воздействия правила.
                /// </summary>
                [XmlArray("ObjectsAffected")]
                [XmlArrayItem("SignalGroupRef")]
                public List<SignalGroupRef> _ObjectsAffected;
                public bool ShouldSerialize_ObjectsAffected()
                {
                    return _ObjectsAffected?.Count > 0;
                }
            }

            /// <summary>
            /// Описание правила задания абсолютного значения задержки.
            /// </summary>
            public class DelayConstant
            {
                /// <summary>
                /// Флаг применения правила.
                /// </summary>
                [XmlAttribute("enabled")]
                public Bool _enabled;
                [XmlIgnore]
                public bool _enabledSpecified { get { return _enabled != Bool.off; } }

                /// <summary>
                /// Параметр правил выравнивания задержек: тип значений констант и допусков.
                /// </summary>
                [XmlAttribute("valueType")]
                public valueType _valueType;

                /// <summary>
                /// Значение константы в правилах выравнивания задержек.
                /// </summary>
                /// <remarks>! Единицы измерения значения зависят от параметра valueType и единиц заданных для всего файла(см.Units).</remarks>
                [XmlAttribute("constant", DataType = "float")]
                public float _constant;

                /// <summary>
                /// Параметр правила выравнивания задержек: нижний допуск.
                /// </summary>
                /// <remarks>! Единицы измерения значения зависят от параметра valueType и единиц заданных для всего файла(см.Units).</remarks>
                [XmlAttribute("toleranceUnder", DataType = "float")]
                public float _toleranceUnder;

                /// <summary>
                /// Параметр правила выравнивания задержек: верхний допуск.
                /// </summary>
                /// <remarks>! Единицы измерения значения зависят от параметра valueType и единиц заданных для всего файла(см.Units).</remarks>
                [XmlAttribute("toleranceOver", DataType = "float")]
                public float _toleranceOver;

                /// <summary>
                /// Объекты воздействия правила.
                /// </summary>
                [XmlArray("ObjectsAffected")]
                [
                    XmlArrayItem("SignalRef", typeof(SignalRef)),
                    XmlArrayItem("DiffSignalRef", typeof(DiffSignalRef)),
                    XmlArrayItem("SignalGroupRef", typeof(SignalGroupRef))
                ]
                public List<object> _ObjectsAffected;
                public bool ShouldSerialize_ObjectsAffected()
                {
                    return _ObjectsAffected?.Count > 0;
                }
            }

            /// <summary>
            /// Описание правила взаимного выравнивания задержек.
            /// </summary>
            /// <remarks>! Правило несимметрично относительно ObjectLeft и ObjectRight</remarks>
            public class DelayRelation
            {
                /// <summary>
                /// Флаг применения правила.
                /// </summary>
                [XmlAttribute("enabled")]
                public Bool _enabled;
                [XmlIgnore]
                public bool _enabledSpecified { get { return _enabled != Bool.off; } }

                /// <summary>
                /// Параметр правил выравнивания задержек: тип значений констант и допусков.
                /// </summary>
                [XmlAttribute("valueType")]
                public valueType _valueType;

                /// <summary>
                /// Значение константы в правилах выравнивания задержек.
                /// </summary>
                /// <remarks>! Единицы измерения значения зависят от параметра valueType и единиц заданных для всего файла(см.Units).</remarks>
                [XmlAttribute("constant", DataType = "float")]
                public float _constant;

                /// <summary>
                /// Параметр правила выравнивания задержек: нижний допуск.
                /// </summary>
                /// <remarks>! Единицы измерения значения зависят от параметра valueType и единиц заданных для всего файла(см.Units).</remarks>
                [XmlAttribute("toleranceUnder", DataType = "float")]
                public float _toleranceUnder;

                /// <summary>
                /// Параметр правила выравнивания задержек: верхний допуск.
                /// </summary>
                /// <remarks>! Единицы измерения значения зависят от параметра valueType и единиц заданных для всего файла(см.Units).</remarks>
                [XmlAttribute("toleranceOver", DataType = "float")]
                public float _toleranceOver;

                /// <summary>
                /// Первый объект воздействия правила взаимного выравнивания задержек.
                /// </summary>
                [XmlElement("ObjectLeft")]
                public ObjectSignal _ObjectLeft;

                /// <summary>
                /// Второй объект воздействия правила взаимного выравнивания задержек.
                /// </summary>
                [XmlElement("ObjectRight")]
                public ObjectSignal _ObjectRight;
            }

            /// <summary>
            /// Правила выравнивания задержек для группы цепей или группы дифференциальных пар.
            /// </summary>
            [XmlElement("DelayEqual")]
            public List<DelayEqual> _DelayEquals;
            public bool ShouldSerialize_DelayEquals()
            {
                return _DelayEquals?.Count > 0;
            }
            /// <summary>
            /// Правила задания абсолютного значения задержки.
            /// </summary>
            [XmlElement("DelayConstant")]
            public List<DelayConstant> _DelayConstants;
            public bool ShouldSerialize_DelayConstants()
            {
                return _DelayConstants?.Count > 0;
            }
            /// <summary>
            /// Правила взаимного выравнивания задержек. 
            /// </summary>
            /// <remarks>! Правила несимметричны относительно ObjectLeft и ObjectRight</remarks>
            [XmlElement("DelayRelation")]
            public List<DelayRelation> _DelayRelations;
            public bool ShouldSerialize_DelayRelations()
            {
                return _DelayRelations?.Count > 0;
            }
        }

        /// <summary>
        /// Настройки поиска сигналов.
        /// </summary>
        public class SignalSearchSettings
        {
            /// <summary>
            /// Правило именования цепей дифференциальных сигналов.
            /// </summary>
            public class RuleDiffSignalNetsNames
            {
                /// <summary>
                /// Флаг применения правила.
                /// </summary>
                [XmlAttribute("enabled")]
                public Bool _enabled;
                [XmlIgnore]
                public bool _enabledSpecified { get { return _enabled != Bool.off; } }

                /// <summary>
                /// Параметр правила именования цепей дифференциальных сигналов: подстрока, определяющая цепь позитивного сигнала.
                /// </summary>
                [XmlAttribute("posStr")]
                public string _posStr;

                /// <summary>
                /// Параметр правила именования цепей дифференциальных сигналов: подстрока, определяющая цепь негативного сигнала.
                /// </summary>
                [XmlAttribute("negStr")]
                public string _negStr;
            }

            /// <summary>
            /// Список цепей, исключённых из поиска сигналов.
            /// </summary>
            public class ExcludedNets
            {
                /// <summary>
                /// Минимальное количество контактов в силовой цепи. Параметр используется для автоматического определения силовых цепей.
                /// </summary>
                [XmlAttribute("minPinsNumber", DataType = "int")]
                public int _minPinsNumber;

                /// <summary>
                /// Cсылки на цепи.
                /// </summary>
                [XmlElement("NetRef")]
                public List<NetRef> _NetRefs;
                public bool ShouldSerialize_NetRefs()
                {
                    return _NetRefs?.Count > 0;
                }
            }

            /// <summary>
            /// Максимальное число цепей в сигнальном кластере. Параметр используется при автоматическом определении цепей сигнального кластера.
            /// </summary>
            [XmlAttribute("maxNetsInCluster", DataType = "int")]
            public int _maxNetsInCluster;

            /// <summary>
            /// Автоматически задавать связи.
            /// </summary>
            [XmlAttribute("createPinPairs")]
            public Bool _createPinPairs;
            [XmlIgnore]
            public bool _createPinPairsSpecified { get { return _createPinPairs != Bool.off; } }

            /// <summary>
            /// Правила именования цепей дифференциальных сигналов.
            /// </summary>
            /// <remarks>! Порядок следования правил в этой секции определяет приоритет правил. Правила следуют в порядке убывания приоритета.</remarks>
            [XmlArray("RulesDiffSignalNetsNames")]
            [XmlArrayItem("RuleDiffSignalNetsNames")]
            public List<RuleDiffSignalNetsNames> _RulesDiffSignalNetsNames;
            public bool ShouldSerialize_RulesDiffSignalNetsNames()
            {
                return _RulesDiffSignalNetsNames?.Count > 0;
            }
            /// <summary>
            /// Список цепей, исключённых из поиска сигналов.
            /// </summary>
            [XmlElement("ExcludedNets")]
            public ExcludedNets _ExcludedNets;
        }

        /// <summary>
        /// Версия раздела.
        /// </summary>
        [XmlAttribute("version")]
        public string _version;

        /// <summary>
        /// Волновые сопротивления и правила разводки сигналов.
        /// </summary>
        [XmlArray("RulesImpedances")]
        [
            XmlArrayItem("Impedance", typeof(Impedance)),
            XmlArrayItem("ImpedanceDiff", typeof(ImpedanceDiff))
        ]
        public List<object> _RulesImpedances;
        public bool ShouldSerialize_RulesImpedances()
        {
            return _RulesImpedances?.Count > 0;
        }
        /// <summary>
        /// Сигнальные кластеры цепей.
        /// </summary>
        [XmlArray("SignalClusters")]
        [XmlArrayItem("SignalCluster")]
        public List<SignalCluster> _SignalClusters;
        public bool ShouldSerialize_SignalClusters()
        {
            return _SignalClusters?.Count > 0;
        }
        /// <summary>
        /// Дифференциальные сигналы.
        /// </summary>
        [XmlArray("DiffSignals")]
        [XmlArrayItem("DiffSignal")]
        public List<DiffSignal> _DiffSignals;
        public bool ShouldSerialize_DiffSignals()
        {
            return _DiffSignals?.Count > 0;
        }
        /// <summary>
        /// Группы сигналов.
        /// </summary>
        [XmlArray("SignalGroups")]
        [XmlArrayItem("SignalGroup")]
        public List<SignalGroup> _SignalGroups;
        public bool ShouldSerialize_SignalGroups()
        {
            return _SignalGroups?.Count > 0;
        }
        /// <summary>
        /// Правила выравнивания задержек.
        /// </summary>
        [XmlElement("RulesDelay")]
        public RulesDelay _RulesDelay;

        /// <summary>
        /// Настройки автоматического поиска сигналов.
        /// </summary>
        [XmlElement("SignalSearchSettings")]
        public SignalSearchSettings _SignalSearchSettings;


        /************************************************************************
         * Здесь находятся функции для работы с элементами класса HiSpeedRules. *
         * Они не являются частью формата TopoR PCB.                            *
         * **********************************************************************/

        /// <summary>
        /// Переименование ссылок на компонент
        /// </summary>
        /// <param name="oldname">старое имя компонента</param>
        /// <param name="newname">новое имя компонента</param>
        /// <summary>m>
        public void Rename_compName(string oldname, string newname)
        {
            if (newname == "" || _SignalClusters == null)
            {
                return;
            }
            foreach (var a in _SignalClusters)
            {
                if (a._SourcePinRef._compName == oldname)
                {
                    a._SourcePinRef._compName = newname;
                }
                if (a._Signals != null)
                {
                    foreach (var b in a._Signals)
                    {
                        if (b._ReceiverPinRef._compName == oldname)
                        {
                            b._ReceiverPinRef._compName = newname;
                        }
                        foreach (var c in b._Components?.Where(r => r._ReferenceName == oldname))
                        {
                            c._ReferenceName = newname;
                        }
                    }
                }
                if (a._PinPairs != null)
                {
                    foreach (var b in a._PinPairs)
                    {
                        foreach (var c in b._PinRefs?.Where(r => r._compName == oldname))
                        {
                            c._compName = newname;
                        }
                    }
                }
            }
        }
        //TODO: разобраться со смыслом существования сигнала при удалении компонента
        /// <summary>
        /// Проверяет, есть сигнал с указанным именем в SignalClusters
        /// </summary>
        /// <param name="name">Имя сигнала</param>
        /// <returns>true - если сигнал существует</returns>
        public bool SignalClusterContain(string name)
        {
            return _SignalClusters?.Where(a => a._Signals?.Where(aa => aa._name == name).Count() > 0).Count() > 0;
        }

        public void Remove_compName(string name)
        {

            bool _ifremove(object o)
            {
                if (o is DiffSignalRef)
                {
                    return !SignalClusterContain((o as DiffSignalRef)._ReferenceName);
                }
                if (o is SignalRef)
                {
                    return !SignalClusterContain((o as SignalRef)._ReferenceName);
                }
                return false;
            }


            if (_SignalClusters != null)
            {
                _SignalClusters.RemoveAll(a => a._SourcePinRef._compName == name);
                foreach (var a in _SignalClusters)
                {
                    a._Signals?.RemoveAll(b => b._ReceiverPinRef._compName == name);
                    foreach (var b in a._Signals?.Where(bb => bb._Components != null))
                    {
                        b._Components.RemoveAll(c => c._ReferenceName == name);
                    }
                    if (a._PinPairs != null)
                    {
                        foreach (var b in a._PinPairs.Where(bb => bb._PinRefs != null))
                        {
                            b._PinRefs.RemoveAll(c => c._compName == name);
                        }
                        if (a._PinPairs.Count < 2)
                        {
                            a._PinPairs.Clear();
                        }
                    }
                }
                if (_SignalGroups != null)
                {
                    foreach (var a in _SignalGroups)
                    {
                        a._References?.RemoveAll(aa => _ifremove(aa));
                    }
                }
            }

        }
        /***********************************************************************/
    }
}
