﻿/* Мною, Константином aka KilkennyCat, 05 июля 2020 года создано сиё 
 * на основе "Описание формата TopoR PCB версия 1.2.0 Апрель 2017 г.".
 * k@kilkennycat.pro
 * http://kilkennycat.ru  http://kilkennycat.pro
 */

using System.Xml.Serialization;

namespace TopoR_PCB_Classes
{
    /// <summary>
    /// Корневой тег. Включает все разделы файла.
    /// </summary>
    [XmlRoot("TopoR_PCB_File")]
    public class TopoR_PCB_File
    {
        /// <summary>
        /// Раздел «Заголовок файла».
        /// </summary>
        [XmlElement("Header")]
        public Header _Header;

        /// <summary>
        /// Раздел «Слои». (Обязательный раздел)
        /// </summary>
        [XmlElement("Layers")]
        public Layers _Layers;

        /// <summary>
        /// Раздел «Стили надписей».
        /// </summary>
        [XmlElement("TextStyles")]
        public TextStyles _TextStyles;

        /// <summary>
        /// Раздел «Библиотечные элементы». (Обязательный раздел)
        /// </summary>
        [XmlElement("LocalLibrary")]
        public LocalLibrary _LocalLibrary;

        /// <summary>
        /// Раздел «Конструктив платы».
        /// </summary>
        [XmlElement("Constructive")]
        public Constructive _Constructive;

        /// <summary>
        /// Раздел «Компоненты на плате». (Обязательный раздел).
        /// </summary>
        [XmlElement("ComponentsOnBoard")]
        public ComponentsOnBoard _ComponentsOnBoard;

        /// <summary>
        /// Раздел «Текущий список соединений».
        /// </summary>
        [XmlElement("NetList")]
        public NetList _NetList;

        /// <summary>
        /// Раздел «Группировка объектов».
        /// </summary>
        [XmlElement("Groups")]
        public Groups _Groups;

        /// <summary>
        /// Раздел «Правила для высокоскоростных устройств».
        /// </summary>
        [XmlElement("HiSpeedRules")]
        public HiSpeedRules _HiSpeedRules;

        /// <summary>
        /// Раздел «Правила».
        /// </summary>
        /// <remarks>! Порядок следования правил в каждой секции определяет приоритет правил. Чем выше приоритет у правила, тем ниже оно описано.</remarks>
        [XmlElement("Rules")]
        public Rules _Rules;

        /// <summary>
        /// Раздел «Соединения на плате».
        /// </summary>
        /// <remarks>В этом разделе описывается конкретная реализация соединений: печатные проводники, межслойные переходы и области металлизации.</remarks>
        [XmlElement("Connectivity")]
        public Connectivity _Connectivity;

        /// <summary>
        /// Раздел «Настройки дизайна».
        /// </summary>
        [XmlElement("Settings")]
        public Settings _Settings;

        /// <summary>
        /// Раздел «Настройки отображения».
        /// </summary>

        [XmlElement("DisplayControl")]
        public DisplayControl _DisplayControl;

        /// <summary>
        /// Раздел «Настройки диалогов».
        /// </summary>
        [XmlElement("DialogSettings")]
        public DialogSettings _DialogSettings;


        /**************************************************************************
         * Здесь находятся функции для работы с элементами класса TopoR_PCB_File. *
         * Они не являются частью формата TopoR PCB.                              *
         * ************************************************************************/

        public void AddPackageFromLibrary(LocalLibrary fromLibrary, dist fromDist, LocalLibrary.Package package)
        {
            _LocalLibrary.AddPackage(fromLibrary, fromDist, _Header._Units._dist, package);
        }

        /// <summary>
        /// Добавление компонента
        /// </summary>
        /// <param name="name">Имя нового компонента. Если имя неуникально, будет добавлен префикс _</param>
        /// <param name="package">упаковка</param>
        public void AddComponent(string name, LocalLibrary.Package package)
        {
            _ComponentsOnBoard?.AddComponent(name, _Header._Units._dist, package);
        }

        /// <summary>
        /// Удаление компонента по имени
        /// </summary>
        /// <param name="name">уникальный имя компонента</param>
        /// <returns>true - если было произведено удаление, иначе (компонент не найден) - false</returns>
        public bool RemoveComponent(string name)
        {
            if (_ComponentsOnBoard.RemoveComponent(name))
            {
                _NetList?.Remove_compName(name);
                _Groups?.Remove_compName(name);
                _Rules?.Remove_compName(name);
                _HiSpeedRules?.Remove_compName(name);
                return true;
            }
            return false;
        }

        /// <summary>
        /// Переименование компонента
        /// </summary>
        /// <param name="oldname">старое имя компонента</param>
        /// <param name="newname">новое имя компонента</param>
        public void RenameComponent(string oldname, string newname)
        {
            _ComponentsOnBoard.RenameComponent(oldname, newname);
            _NetList?.Rename_compName(oldname, newname);
            _Groups?.Rename_compName(oldname, newname);
            _Rules?.Rename_compName(oldname, newname);
            _HiSpeedRules?.Rename_compName(oldname, newname);
        }

        /**************************************************************************/
    }//End of class



}
